/*=============================================================================
	Fragmentshader for DrawComplexSurface, single pass.

	Revision history:
		* Created by Smirftsch
=============================================================================*/

uniform sampler2D Texture0;	//Base Texture
uniform sampler2D Texture1;	//Lightmap
uniform sampler2D Texture2;	//DetailTexture
uniform sampler2D Texture3;	//MacroTexture
uniform sampler2D Texture4;	//BumpMap
uniform sampler2D Texture5;	//FogMap
uniform sampler2D Texture6;	//EnvironmentMap
uniform float AlphaThreshold;

uniform vec4 DrawColor;
uniform bool[6] ComplexAdds;
uniform uint PolyFlags;
uniform uint RendMap;
uniform bool bHitTesting;
uniform float Gamma;

uniform bool FogChanged;

in vec3 vCoords;
in vec2 vTexCoords;
in vec4 vEyeSpacePos;
in vec2 vLightMapCoords;
in vec2 vDetailTexCoords;
in vec2 vMacroTexCoords;
in vec2 vBumpTexCoords;
in vec2 vFogMapCoords;
in vec2 vEnvironmentTexCoords;
in vec4 vLightPos[8];

//in vec3 gTangent;
//in vec3 gBitangent;
//in vec3 gNormals;

#ifdef GL_ES
out vec4 FragColor;
#else
layout ( location = 0, index = 0) out vec4 FragColor;
//layout ( location = 0, index = 1) out vec4 FragColor1;
#endif

uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Oonly for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));

	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);

	return FogResult;
}

vec3 rgb2hsv(vec3 c)
{
	// some nice stuff from http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main (void)
{
	vec4 TotalColor;
	vec4 Color = texture(Texture0, vTexCoords);

	// Handle PF_Masked.
	if ( (PolyFlags&PF_Masked) == PF_Masked )
	{
		if(Color.a < 0.5)
			discard;
		else Color.rgb /= Color.a;
	}
	else if ( (PolyFlags&PF_AlphaBlend) == PF_AlphaBlend && Color.a < 0.01 )
		discard;

	TotalColor=Color;

	// LightMap
	if (ComplexAdds[0] == true)
	{
		vec4 LightColor = texture(Texture1, vLightMapCoords);
		TotalColor*=vec4(LightColor.rgb,1.0);
		TotalColor=clamp(TotalColor*2.0,0.0,1.0); //saturate.
	}

	// DetailTextures
	float bNear = clamp(1.0-(vCoords.z/380.0),0.0,1.0);
	if (ComplexAdds[1] == true && bNear > 0.0)
	{
		vec4 DetailTexColor = texture(Texture2, vDetailTexCoords);
		vec3 hsvDetailTex = rgb2hsv(DetailTexColor.rgb); // cool idea Han :)
		hsvDetailTex.b += (DetailTexColor.r - 0.1);
		hsvDetailTex = hsv2rgb(hsvDetailTex);
		DetailTexColor=vec4(hsvDetailTex,0.0);
		DetailTexColor = mix(vec4(1.0,1.0,1.0,1.0), DetailTexColor, bNear); //fading out.
		TotalColor*=DetailTexColor;
	}

	// MacroTextures
	if (ComplexAdds[2] == true)
	{
		vec4 MacrotexColor = texture(Texture3, vMacroTexCoords);
		vec3 hsvMacroTex = rgb2hsv(MacrotexColor.rgb);
		hsvMacroTex.b += (MacrotexColor.r - 0.1);
		hsvMacroTex = hsv2rgb(hsvMacroTex);
		MacrotexColor=vec4(hsvMacroTex,1.0);
		TotalColor*=MacrotexColor;
	}

	// (still yet not really real) BumpMap
	if (ComplexAdds[3] == true)
	{
		//normal from normal map
		vec3 normal = normalize(texture(Texture4, vBumpTexCoords).rgb * 2.0 - 1.0);

		vec3 BumpColor;
		vec3 TotalBumbColor=vec3(0.0,0.0,0.0);
		for(int i=0; i<8; ++i)
		{
			vec4 LightPos = vLightPos[i];

			if (LightPos.w == 0.0)
				continue;

			if ( ((PolyFlags&PF_Unlit) == PF_Unlit) || ((PolyFlags&PF_BumpMap) == PF_BumpMap))
			{
				LightPos=vec4(1.0,1.0,1.0,1.0);

				//Lighting diffuse value
				float diffuse = max(dot(normal, LightPos.xyz), 0.0);

				BumpColor = diffuse * texture(Texture0, vBumpTexCoords).rgb;
				TotalBumbColor = BumpColor;
				break;
			}
			else if (LightPos.w == 0.0)
				break;

			LightPos.xyz = normalize(LightPos.xyz);

			//Lighting diffuse value
			float diffuse = max(dot(normal, LightPos.xyz), 0.0);

			BumpColor = diffuse * texture(Texture0, vBumpTexCoords).rgb;
			TotalBumbColor += BumpColor;
		}
		if (TotalBumbColor.x != 0.0 || TotalBumbColor.y != 0.0 || TotalBumbColor.z != 0.0) //no light close enough.
			TotalColor*=vec4(clamp(TotalBumbColor,0.0,1.0),1.0);
	}

	// FogMap
	if (ComplexAdds[4] == true)
	{
		vec4 FogColor = texture(Texture5, vFogMapCoords)*2.0;
		TotalColor.rgb = TotalColor.rgb * (1.0-FogColor.a) + FogColor.rgb;
		TotalColor.a   = FogColor.a;
	}

	// Environmentmap
	if (ComplexAdds[5] == true)
	{
		vec4 EnvironmentColor = texture(Texture6, vEnvironmentTexCoords);
		TotalColor*=vec4(EnvironmentColor.rgb,1.0);
	}

	TotalColor=clamp(TotalColor,0.0,1.0); //saturate.

	// Add distance fog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		vec4 FogColor = FogParams.FogColor;
		if ( (PolyFlags&PF_Modulated) == PF_Modulated )
			FogColor = vec4(0.5,0.5,0.5,0.0);
		else if ( (PolyFlags&PF_Translucent) == PF_Translucent )
			FogColor = vec4(0.0,0.0,0.0,0.0);

		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(TotalColor, FogColor, getFogFactor(FogParams, FogCoord));
	}

	if((PolyFlags & PF_Modulated)!=PF_Modulated)
	{
		// Gamma
		TotalColor.r=pow(TotalColor.r,1.8-Gamma);
		TotalColor.g=pow(TotalColor.g,1.8-Gamma);
		TotalColor.b=pow(TotalColor.b,1.8-Gamma);
	}

#ifdef EDITOR
	// Editor support.
	if ( (PolyFlags&PF_Selected) == PF_Selected )
	{
		TotalColor.r = (TotalColor.r*0.125);
		TotalColor.g = (TotalColor.g*0.125);
		TotalColor.b = (TotalColor.b*0.125) + 0.5;
		TotalColor = clamp(TotalColor,0.0,1.0);
		if(TotalColor.a < 0.5)
			TotalColor.a = 0.51;
	}
	// HitSelection, Zoneview etc.
	if (bHitTesting)
		TotalColor = DrawColor; // Use ONLY DrawColor.
	else if (RendMap == REN_Zones || RendMap == REN_PolyCuts || RendMap == REN_Polys)
	{
		TotalColor +=0.5;
		TotalColor *= DrawColor;
	}
#endif

	//FragColor1	= mix(TotalColor,LightColor,1.0);// way to fix skybox etc??
	FragColor	= TotalColor;
}

/*
//
// EnviroMap.
//
// Simple GLSL implementation of the C++ code. Should be obsoleted by some fancy
// per pixel sphere mapping implementation. But for now, just use this approach
// as PF_Environment handling is the last missing peace on obsoleting RenderSubsurface.
//
vec2 EnviroMap( vec3 Point, vec3 Normal )
{
	vec3 R = reflect(normalize(Point),Normal);
	return vec2(0.5*dot(R,Uncoords_XAxis)+0.5,0.5*dot(R,Uncoords_YAxis)+0.5);
}
*/