/*=============================================================================
	Fragmentshader for DrawGouraudPolygon, in 227 also DrawGouraudPolygonList.

	Revision history:
		* Created by Smirftsch
=============================================================================*/

uniform sampler2D Texture0;	//Base Texture
uniform sampler2D Texture1;	//DetailTexture

uniform float AlphaThreshold;
uniform vec4 DrawColor;
uniform uint PolyFlags; 
uniform bool bHitTesting;
uniform float Gamma;

uniform bool DrawGouraudFog;//VertexFog

// Texture UV's
uniform vec2 DrawGouraudTexUV;
uniform vec3 DrawGouraudDetailTexUV;

in vec3 vCoords;
in vec4 vEyeSpacePos;
in vec4 vLightColor;
in vec4 vFogColor; //VertexFog
in vec2 vTexCoords;

out vec4 FragColor;

//DistanceFog
uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Only for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));
		
	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);
	
	return FogResult;
}

vec3 rgb2hsv(vec3 c)
{
	// some nice stuff from http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main(void)
{
	vec4 TotalColor;
	vec4 Color = texture(Texture0, (vTexCoords*DrawGouraudTexUV)); 
		
	// Handle PF_Masked.
	if ( (PolyFlags&PF_Masked) == PF_Masked )
	{
		if(Color.a < 0.5)
			discard;
		else Color.rgb /= Color.a;
	}
	else if ( (PolyFlags&PF_AlphaBlend) == PF_AlphaBlend && Color.a < 0.01 )
		discard;

	// Handle PF_RenderFog.
	if ((PolyFlags&PF_RenderFog) == PF_RenderFog)
	{
		// Handle PF_RenderFog|PF_Modulated.
		if ( (PolyFlags&PF_Modulated)== PF_Modulated )
		{
			// Compute delta to modulation identity.
			vec3 Delta = vec3(0.5) - Color.xyz;
			
			// Reduce delta by (squared) fog intensity.
			//Delta *= 1.0 - sqrt(vFogColor.a);
			Delta *= 1.0 - vFogColor.a;
			Delta *= vec3(1.0) - vFogColor.rgb;
			
			TotalColor = vec4(vec3(0.5)-Delta,Color.a);
		}
		// Normal.
		else
		{
			Color*=vLightColor;
			//TotalColor=mix(Color, vec4(vFogColor.xyz,1.0), vFogColor.w);
			TotalColor.rgb = Color.rgb * (1.0-vFogColor.a) + vFogColor.rgb;
			TotalColor.a   = Color.a;
		}

	}
	// No Fog.
	else if((PolyFlags & PF_Modulated) == PF_Modulated)
	{
		TotalColor = Color;
	}
	else
	{
		TotalColor = Color * vec4(vLightColor.rgb,1.0);
	}
	float bNear = clamp(1.0-(vCoords.z/380.0),0.0,1.0);
	if(DrawGouraudDetailTexUV.z > 0.0 && bNear > 0.0)
	{
		vec4 DetailTexColor = texture(Texture1, (vTexCoords*DrawGouraudDetailTexUV.xy)); // DetailTexture
		vec3 hsvDetailTex = rgb2hsv(DetailTexColor.rgb);
		hsvDetailTex.b += (DetailTexColor.r - 0.1);
		hsvDetailTex = hsv2rgb(hsvDetailTex);
		DetailTexColor=vec4(hsvDetailTex,0.0);
		DetailTexColor = mix(vec4(1.0,1.0,1.0,1.0), DetailTexColor, bNear); //fading out.
		TotalColor*=DetailTexColor;
	}
	
	// Add DistanceFog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		vec4 FogColor = FogParams.FogColor;
		if ( (PolyFlags&PF_Modulated) == PF_Modulated )
			FogColor = vec4(0.5,0.5,0.5,0.0);
		else if ( (PolyFlags&PF_Translucent) == PF_Translucent )
			FogColor = vec4(0.0,0.0,0.0,0.0);
			
		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(TotalColor, FogColor, getFogFactor(FogParams, FogCoord));

		if((PolyFlags & PF_Modulated)!=PF_Modulated)
		{
			// Gamma
			TotalColor.r=pow(TotalColor.r,1.8-Gamma);
			TotalColor.g=pow(TotalColor.g,1.8-Gamma);
			TotalColor.b=pow(TotalColor.b,1.8-Gamma);
		}
	}
	else if((PolyFlags & PF_Modulated)!=PF_Modulated)
	{
		// Gamma
		TotalColor.r=pow(TotalColor.r,2.2-Gamma);
		TotalColor.g=pow(TotalColor.g,2.2-Gamma);
		TotalColor.b=pow(TotalColor.b,2.2-Gamma);
	}

#ifdef EDITOR
	// Editor support.
	if ( (PolyFlags&PF_Selected) == PF_Selected )
	{
		TotalColor.r = (TotalColor.r*0.125);
		TotalColor.g = (TotalColor.g*0.125);
		TotalColor.b = (TotalColor.b*0.125);
		TotalColor = clamp(TotalColor,0.0,1.0);
		if(TotalColor.a < 0.5)
			TotalColor.a = 0.51;
	}
	// HitSelection, Zoneview etc.
	if (bHitTesting)
		TotalColor = DrawColor; // Use DrawColor.
#endif

	FragColor = TotalColor; 
}

// Blending translation table
/*
//PF_Modulated
//glBlendFunc( GL_DST_COLOR, GL_SRC_COLOR );
//pixel_color * gl_FragColor
	
GL_ONE 						vec4(1.0)
GL_ZERO 					vec4(0.0)
GL_SRC_COLOR 				gl_FragColor
GL_SRC_ALPHA 				vec4(gl_FragColor.a)
GL_DST_COLOR 				pixel_color
GL_DST_ALPHA 				vec4(pixel_color.a)
GL_ONE_MINUS_SRC_COLOR		vec4(1.0) - gl_FragColor
GL_ONE_MINUS_SRC_ALPHA 		vec4(1.0  - gl_FragColor.a)
GL_ONE_MINUS_DST_COLOR 		vec4(1.0) - pixel_color
GL_ONE_MINUS_DST_ALPHA 		vec4(1.0  - pixel_color.a)

if (PolyFlags & PF_Invisible)
	glBlendFunc(GL_ZERO, GL_ONE);

if (PolyFlags & PF_Translucent)
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);

if (PolyFlags & PF_Modulated)
	glBlendFunc(GL_DST_COLOR, GL_SRC_COLOR);

if (PolyFlags & PF_AlphaBlend)
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

if (PolyFlags & PF_Highlighted)
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
*/